#include <fstream>
#include <string>

using namespace std;

#include "GObject.h"
#include "Parser.h"
#include "SNMPObject.h"
#include "SNMPRouter.h"
#include "SNMPConnection.h"
#include "Poll.h"

Parser::Parser(const string &fname)
{
	ifstream ff(fname.c_str());
	int linecounter = 0;
	string k;
	GdkFont *fontgdk;
	
	if(!ff)	{	
		cout<<"Configuration file missing"<<endl;
		exit(0);
    	}
    
	while(!ff.eof())
	{
		ff>>k;
		linecounter ++;
		
		if(!ff) break;
		
		if(k[0] == '#') {
			char garbage[1024];
			ff.getline(garbage,1024);
			continue;
		} else if(k == "background") {
			ff>>_background;
	    		ff>>_width;
			ff>>_height;
		} else if(k == "font") {
	    		ff>>_font;
			fontgdk=gdk_font_load(_font.c_str());
		} else if(k == "geometry") {
			ff>>_width>>_height;
		} else if(k == "poll_interval") {
			ff>>_polldelay;
		} else if(k == "router") {
			string name, host, community;
			vector<Poll *> tmp;
			int x,y;
			ff>>name;
			ff>>k; linecounter++;
			while(k != "}")
			{
				ff>>k; linecounter++;
				if(k == "host")
			    	ff>>host;
				else if(k == "community")
					ff>>community;
				else if(k == "position")
					ff>>x>>y;
				else if(k == "poll")
				{
					string host, comm, path, incrstr;
					int incr; long int max;
					ff>>host>>comm>>path>>incrstr>>max;
					if(incrstr == "gauge")
				    		incr = 0;
					else if(incrstr == "counter")
						incr = 1;
					else
					{
						cout<<"Parse error in config file: line: "<<linecounter<<endl;
						exit(1);
					}
					Poll *tpoll=new Poll(host, comm, path, incr, max);
					tmp.push_back(tpoll);
				}
				else if(k != "}")
				{
					cout<<"Parse error in router entry: line: "<<linecounter<<endl;
					exit(0);
				}
			}
			SNMPRouter *r=new SNMPRouter(name, host, community, x, y, fontgdk);
			for(int i=0; i<tmp.size(); i++)
			r->addPoll(tmp[i]);
			_obj.push_back(r);
		}
		else if(k == "connection")
		{    
			int defaults=1;
			string host, name, community;
			int speed, intno, x1, y1, x2, y2;
			vector<Poll *> tmp;
			ff>>name;
			ff>>k; linecounter++;
			while(k != "}")
			{
				ff>>k; linecounter++;
				if(k == "host")
					ff>>host;
				else if(k == "community")
					ff>>community;
				else if(k == "position")
					ff>>x1>>y1>>x2>>y2;
				else if(k == "interface")
					ff>>intno;
				else if(k == "speed")
					ff>>speed;
				else if(k == "no_default")
					defaults=0;
				else if(k == "poll")
				{
					string host, comm, path, incrstr;
					int incr; long int max;
					ff>>host>>comm>>path>>incr>>max;
       				        if(incrstr == "gauge")
				    	    incr = 0;
				        else if(incrstr == "counter")
				    	    incr = 1;
				        else
				        {
				            cout<<"Parse error in config file: line: "<<linecounter<<endl;
					    exit(1);
				        }

					Poll *tpoll=new Poll(host, comm, path, incr, max);
					tmp.push_back(tpoll);
				}
				else if(k != "}")
				{
					cout<<"Parse error in connection entry: line:"<<linecounter<<endl;
					exit(0);
				}
			}
			SNMPConnection *c=new SNMPConnection(intno, speed, x1, y1, x2, y2, name, host, community, defaults);
			for(int i=0; i<tmp.size(); i++)
			c->addPoll(tmp[i]);
			_obj.push_back(c);
		}
		else if(k != "")
		{
		    cout<<"Parse error in config file: line: "<<linecounter<<endl;
	    	cout<<"Found "<<k<<" unexpected."<<endl;
		    exit(0);
		}
    }
}

const string& Parser::getBackground()
{
	return _background;
}

int Parser::getWidth()
{
	return _width;
}

int Parser::getHeight()
{
	return _height;
}

Parser::~Parser()
{
	for(int x=0; x<_obj.size(); x++)
		delete _obj[x];
}
