#include "general.h"
#include "Poll.h"
#include "snmppoll.h"

#include <cstdio>

Poll::Poll(const string &host, const string &community, const string &path, int incremental, int max)
{
	_host = host;
	_community = community;
	_path = path;
	_incremental = incremental;
	_max = max;
	_lastval = 0;
	_lasttick = 0;
	_lastreal = 0;
}

long long int Poll::getLastRealValue()
{
	return _lastreal;
}

string Poll::getPath()
{
	return _path;
}

int Poll::getValue()
{
	long long int retval;
	long long int ticks;
	
	long long int absval = snmp_get_integer(_host.c_str(), _community.c_str(), _path.c_str() );
	
	if(absval == -1) 
		retval = absval;
	else
	{
		if(_incremental)
		{
			string crttime = snmp_get_string(_host.c_str(), _community.c_str(), "sysUpTime.0");
			if(crttime == "-1")
				retval = -1;
			else
			{
				char tickbuff[255];
				_lastreal = absval - _lastval;
				_lastval = absval;
				ticks = getTicks(crttime);
				long long int elapsed=ticks-_lasttick;
				retval =(long long int)( (_lastreal*10000.0)/elapsed/_max );
				_lasttick=ticks;
				_lastreal = _lastreal*100/elapsed;
			}
		}
		else
		{	
			_lastreal = absval;
			retval = (long long int)((_lastreal*100.0)/_max);
		}
	}
	return retval;
}
