#include <math.h>
#include <strstream>
#include <string>

using namespace std;

#include "GObject.h"
#include "general.h"
#include "Window.h"
#include "SizeW.h"
#include "DrawingArea.h"
#include "SNMPObject.h"
#include "SNMPConnection.h"
#include "snmppoll.h"
#include "Console.h"
#include "Poll.h"

SNMPConnection::SNMPConnection(int number, int speed, int x1, int y1, int x2, 
				int y2, const string &name, const string &host, 
				const string &community,
				int def_load=1 ):
SNMPObject(name, host, community)
{
	_speed = speed;
	_number = number;
	_x1=x1; _y1=y1; _x2=x2; _y2=y2;

	double d, dx, dy;
	d=sqrt((_x2-_x1)*(_x2-_x1)+(_y2-_y1)*(_y2-_y1));
	dx=10*(_x2-_x1)/d;
	dy=10*(_y2-_y1)/d;
	_x1+=int(dx);
	_x2-=int(dx);
	_y1+=int(dy);
	_y2-=int(dy);
	_defaults = def_load;
	
	Poll *in, *out;
	if(_defaults)
	{
		in=new Poll(host, community, "ifInOctets."+ toString(_number), true, _speed);
		out=new Poll(host, community, "ifOutOctets."+ toString(_number), true, _speed);
		assert(in); assert(out);
		addPoll( in );
		addPoll( out );
	}
	_updated = 1;
}

void SNMPConnection::draw(DrawingArea *d)
{
	extern Console *global_console;
	Console *console = global_console;

    if(_value>=0)
    {
		d->line(_x1, _y1, _x2, _y2, 5, 0, 0, 0);
		d->line(_x1, _y1, _x2, _y2, 3, 0, 255, 0);
		double mx=(_x2-_x1)*((float)_value)/100;
		double my=(_y2-_y1)*((float)_value)/100;
		d->line(_x1, _y1, _x1+rint(mx), _y1+rint(my), 3, 255, 0, 0);
		if( _updated )
		{
			string tmp = _host + " : " + toString(_value) + "% reached on " + _name + "\n";
			if(_value<60)
				return;
			else if(_value<80)
				console->addText(tmp, console->_warning); 
			else 
				console->addText(tmp, console->_minor);
		}
    }
    else if(_value==SNMPObject::oper_down)
    {
		d->line(_x1, _y1, _x2, _y2, 5, 0, 0, 0);
		d->line(_x1, _y1, _x2, _y2, 3, 0, 0, 255);
		if( _updated )
		{
			string tmp = _host + " : " + _name + " operational status is down\n";
			console->addText(tmp, console->_fatal);
		}
    }
    else if(_value==SNMPObject::admin_down)
    {
		d->line(_x1, _y1, _x2, _y2, 5, 0, 0, 0);
		d->line(_x1, _y1, _x2, _y2, 3, 255, 255, 0);
		if( _updated )
		{
			string tmp = _host + " : " + _name + " administrative status is down\n";
			console->addText(tmp, console->_major);
		}
    }
    else
    {
		d->line(_x1, _y1, _x2, _y2, 5, 0, 0, 0);
		d->line(_x1, _y1, _x2, _y2, 3, 255, 255, 255);
		if( _updated )
		{
			string tmp = _host + " : Timeout while reading status for " + _name + "\n";
			console->addText(tmp, console->_severe);
		}
    }
	_updated = 0;
}

void SNMPConnection::update()
{
	//la o conexiune este monitorizata implicit starea (up/down)
	//si apoi poll-urile cerute din fisierul de configuratie
	if(_defaults)
	{
		string adminstr, operstr;
		operstr = "ifOperStatus." + toString(_number);
		adminstr = "ifAdminStatus." + toString(_number);
		string oper_status=snmp_get_string(_host.c_str(), _community.c_str(), operstr.c_str() );
		string admin_status=snmp_get_string(_host.c_str(), _community.c_str(), adminstr.c_str() );
		if(admin_status == "down(2)")
			_value=SNMPObject::admin_down;
		else if(oper_status == "down(2)")
			_value=SNMPObject::oper_down;
		else
			_value=getWorst();
	} else {
		_value = getWorst();
	}
	
	_updated = 1;
}

bool SNMPConnection::inRange(double x, double y)
{
	if ( _x2 == _x1 )
	{
		if((_y1 <= y && y <= _y2) || (_y2 <= y && y <= _y1))
		{
			if(fabs(x - _x1) <= 3)
				return true;
			else
				return false;
		}
		else
			return false;
	}

	if ( _y2 == _y1 )
	{
		if((_x1 <= x && x <= _x2) || (_x2 <= x && x <= _x1))
		{
			if(fabs(y - _y1) <= 3)
				return true;
			else
				return false;
		}
		else 
			return false;
	}

	double m = ((double)_y2 - _y1)/(_x2 - _x1);
	double n = ((double)_x1 * _y2 - _x2 * _y1) / (_x1 - _x2);
	double d = fabs(m * x - y + n) / sqrt(m*m + 1);
	
	if( d <= 3 )
	{
		double xm = (m*y + x - m*n) / (m*m +1);
		if(( _x1 <= xm && xm <= _x2 ) || ( _x2 <= xm && xm <= _x1 ))
			return true;
		else
			return false;
	}
	else
		return false;
}


string SNMPConnection::getState()
{
	string retval = _host + " / " + _name + ": ";
	for(int x=0; x<_poll.size(); x++)
	{
		long long int val = _poll[x]->getLastRealValue();
		retval = retval + "(" + _poll[x]->getPath() + ": " + toString(val) + "/ ";
		if(val > 1024)
		{
			val /= 1024;
			retval = retval + toString(val) + "Kb/s/ ";
			val *= 8;
			retval = retval + toString(val) + "Kbps) ";
		}
		else
		{
			val = val * 8;
			retval = retval + toString(val) + "bps) ";
		}
	}
	return retval;
}
